/**
 * Define a message resolver
 * @function resolver
 * @returns {function}
 */
'use strict'

/** @lends resolver */
function resolver (messages) {
  /**
   * Resolver function
   * @param {string} keypath - Keypath to resolve
   */
  return function definedResolver (keypath, options = {}) {
    let { formatter = (message) => message } = options
    if (!keypath) {
      return keypath
    }
    if (typeof keypath === 'object') {
      return Object.keys(keypath)
        .filter((key) => !/^\$/.test(key))
        .reduce((resolved, key) => Object.assign(resolved, {
          [key]: definedResolver(keypath[ key ], {
            formatter: (message) => formatter(message, key)
          })
        }), {})
    }
    let keys = keypath.split(/\./)
    let data = messages
    while (data && keys.length > 0) {
      let key = keys.shift()
      let message = data[ key ]
      if (typeof message === 'string') {
        return formatter(message)
      }
      data = message
    }
    console.warn(`Message not found with keypath: ${keypath}`)
  }
}

Object.assign(resolver, {
  /**
   * Create a multiple resolvers
   * @param messages
   * @returns {Array}
   */
  combine (messages) {
    return Object.keys(messages)
      .reduce((defined, lang) => Object.assign(defined, {
        [lang]: resolver(messages[ lang ])
      }), {})
  }
})

module.exports = resolver
